*Examining Publication Bias – A Simulation-Based Evaluation of Statistical Tests on Publication Bias
*Andreas Schneck

*last checked 19.06.2017

set dp period
cap log close
set matsize 1000
set scheme burd


log using simul, replace

*power detecting effect for studies in MA
 power onecorrelation 0 0.05, n(164) // small effect small n
 scalar lnle =  r(power) 
 power onecorrelation 0 0.1, n(164) // medium effect small n
 scalar lnme =  r(power) 
 power onecorrelation 0 0.15, n(164) // medium effect small n
 scalar lnhe =  r(power) 
 power onecorrelation 0 0.05, n(500) // small effect small n
  scalar hnle =  r(power) 
 power onecorrelation 0 0.1, n(500) // medium effect small n
   scalar hnme =  r(power) 
 power onecorrelation 0 0.15, n(500) // medium effect small n
   scalar hnhe =  r(power)
  
  di (lnle+lnme+lnhe+hnle+hnme+hnhe) /6


clear
use "sim1.dta", clear
append using "sim2.dta", gen(run)
append using "sim3.dta", gen(run2)
append using "sim4.dta", gen(run3)
replace run = run2+1 if run==. & run2!=.
replace run = run3+2 if run==. & run2==. & run3!=.
fre run

drop run2 run3
mvdecode _all, mv(-999=.) 


sum reg_count, detail
scalar rc = r(sum)
di rc

sum k
di r(sum)

sum r_time if run==0, detail
scalar rt = r(sum)
di rt/60/60

scalar rps = rc/rt
di rps

replace het = het/100
*gen es_n = es
replace es = 2 if es ==1
replace es = 1 if es == 0.5
replace es = 3 if es == 1.5
replace es = 4 if het ==1
label define es 0"es = 0" 1"es = 0.5" 2"es = 1" 3"es = 1.5" 4"es = het [0;1.5]"
label value es es

replace I_sq = 0 if I_sq<0



save basedata.dta, replace

gen fat_p = ttail(k, fat_t)

foreach var of varlist pu_p fat_p tes_p ct3_p ct5_p ct10_p ct15_p {
gen `var'_sig = .
replace `var'_sig=0 if `var'>=0.05 & `var'!=.
replace `var'_sig=1 if `var'<0.05
}

bysort het: sum I_sq, detail
gen pb_p = pb_suc/pb_com
gen pb_c = pb_com/k
gen pb_s =  pb_suc/k
bysort pb: sum pb_p pb_c pb_s, detail

bysort pb trim: sum pb_p pb_s, detail

di 12.5/21.7
di 25/43

gen pb_com_c = pb_c*10
gen pb_suc_c = pb_s*10

sum pb_com_c pb_suc_c if pb!=0

replace pb_suc_c = 0 if pb_com==0

sum pb_com_c if pb !=0, detail
replace pb_com_c = (pb_com_c-r(mean))
sum pb_suc_c if pb !=0, detail
replace pb_suc_c = (pb_suc_c-r(mean))

replace I_sq = I_sq*10

************************
*regression results
capture erase p_res.doc
reg ct3_p if pb!=0 
foreach var of varlist pu_p fat_p tes_p ct3_p ct5_p ct10_p ct15_p {
reg `var'_sig i.k I_sq i.trim pb_com_c pb_suc_c if pb!=0
test _b[_cons] = 0.8
outreg2 using p_res.doc, append se dec(3)
}

capture erase a_res.doc
foreach var of varlist pu_p fat_p tes_p ct3_p ct5_p ct10_p ct15_p {
reg `var'_sig i.k I_sq if pb==0
test _b[_cons] = 0.05
outreg2 using a_res.doc, append se dec(3)
}

************************
*descriptive statistics
matrix res_power = J(80,14,.)
local rcount = 1
local rowname = ""
local colname = ""
forvalues tr = 0(1)1 {
forvalues pb = 50(50)100 {
forvalues n = 100(400)500 {
forvalues k = 100(900)1000 {
forvalues es = 0(1)4 {
local temp = "tr`tr'pb`pb'n`n'k`k'es`es'"
local rowname = "`rowname' `temp'"
local ccount = 1
foreach var of varlist pu_p fat_p tes_p ct3_p ct5_p ct10_p ct15_p {
sum `var'_sig if trim==`tr' & pb==`pb' & mu_n==`n' & k==`k' & es==`es'
matrix res_power[`rcount',`ccount'] = r(mean)
local ccount = `ccount'+1
if `rcount'==1 {
local ctemp = "`var'_s `var'_p" 
local colname = "`colname' `ctemp'"
}
bitest `var'_sig == 0.8 if trim==`tr' & pb==`pb' & mu_n==`n' & k==`k' & es==`es'
matrix res_power[`rcount',`ccount'] = r(p)
local ccount =  `ccount'+1

} 
local rcount = `rcount'+1
}
}
}
}
}

matrix rownames res_power =`rowname'
matrix colnames res_power =`colname'

putexcel set results, sheet(power, replace) modify
putexcel A1=matrix(res_power),  names

*alpha
matrix res_alpha = J(20,14,.)
local rcount = 1
local rowname = ""
local colname = ""

forvalues n = 100(400)500 {
forvalues k = 100(900)1000 {
forvalues es = 0(1)4 {
local temp = "n`n'k`k'es`es'"
local rowname = "`rowname' `temp'"

local ccount = 1
foreach var of varlist pu_p fat_p tes_p ct3_p ct5_p ct10_p ct15_p {
sum `var'_sig if  pb==0 & mu_n==`n' & k==`k' & es==`es'
matrix res_alpha[`rcount',`ccount'] = r(mean)
local ccount =  `ccount'+1

if `rcount'==1 {
local ctemp = "`var'_s `var'_p" 
local colname = "`colname' `ctemp'"
}

bitest `var'_sig == 0.05 if  pb==0 & mu_n==`n' & k==`k' & es==`es'
matrix res_alpha[`rcount',`ccount'] = r(p)
local ccount =  `ccount'+1
} 
local rcount = `rcount'+1
}
}
}
di "`rowname'"
matrix rownames res_alpha =`rowname'
matrix colnames res_alpha =`colname'
putexcel set results, sheet(alpha, replace) modify
putexcel A1=matrix(res_alpha),  names
*************************************
*************************************
log close
