*Examining Publication Bias – A Simulation-Based Evaluation of Statistical Tests on Publication Bias
*Andreas Schneck

*last checked 19.06.2017

clear
set matsize 10000
macro drop _all

*********************
ssc install metafunnel 
do fastreg.do
do clear_char.do	
do sim_process.do

matrix regs = J(1000,7,.)
matrix colnames regs = koef se N df pb_a pb_c pb


global run = 1
global hetrange = 2
global sigma_x = 2 //also sigma z
global sigma_e = 10
global sigma_z = 10
global sigma_n = 150

global coefs = 3
global ds = 9
global run2 = 1
tuples z1 z2 z3
global ntuples = `ntuples'
forvalues sreg = 1(1)$ntuples {
global tuple`sreg' = "`tuple`sreg''"
}

scalar reg_count=1

foreach trimming of numlist 0 1 {
if `trimming' ==1 {
local pbias=100
}

else {
local pbias=0
}

dset, es(0.5) pb(`pbias') trim(0) k(1000) mu_n(100) het(0)
clear
svmat regs, names(col)		//load meta-results in stata


gen t = koef/se
gen p = 2*(ttail(df,abs(t)))	//two-sided p-value
gen p_os = ttail(df,t)			// one-sided
*local pb = `pb'
sum pb_a						
scalar k = r(N)					//if publication bias tendency
scalar pb_int = r(sum)
di pb_int
sum N
scalar mean_N = r(mean)


*publication bias behaviour/intention
sum pb_c
local pb_com = r(sum)			//if publication bias comitted (opportunity)
di `pb_com'

sum pb_c if t>1.96 & pb_c==1
local pb_suc = r(N)
di `pb_suc'

*local het = `het'
*local n = `mu_n'
*local es = `es'


*p-curve
// van Assen, M. A. L. M., van Aert, R. C. M., & Wicherts, J. M. (2015). Meta-analysis using effect size distributions of only statistically significant studies. Psychological Methods, 20(3), 293-309.
// p.297

gen w = 1/se^2
reg koef [aweight=w]

*I2 Heterogeneity Thompson & Higgins 2002
predict res, resid
replace res = w*res^2
sum res
local Isq = (r(sum)-(r(N)-1))/r(sum)

local fe =  _b[_cons]			//fixed effects meta-analysis to compare estimate to
drop w

sum p	if p<0.05		
if r(N)==0 {					//missing if p<0.05 missing	
local pc_dfrs = -999
local pc_prs = -999
}

else {
gen tcv = invttail(df, 0.025) 	//study specific significance threshold

gen CV = tcv * se
gen pmarg = 1-normal((CV-`fe')/se)	//significance threshold in relation to mean effect
gen ph1 = 	1-normal((koef-`fe')/se)	//effect related to mean effect
replace ph1 = 1-ph1 if koef<0
gen q = ph1/pmarg if p<0.05

gen pp = q
replace q = log(q)
sum q
scalar sumq = -r(sum)

local pu_N = r(N)
local pu_p = gammap(r(N), sumq)	//Fishers' method but gamma distribution with k,1 df
scalar drop sumq
di `pu_N'
di `pu_p'

hist pp, width(0.05) freq xtitle("pp-value") title("`pbias'% Publication Bias") name("pu_`trimming'", replace)
}


*************************

*Funnel Plot (new)
metafunnel koef se, xtitle("Effect") ytitle("Standard Error of Effect") title("`pbias'% Publication Bias") name("funnel_`trimming'", replace) subtitle(" ")

*FAT_PET
gen prec = 1/se
reg t prec		//FAT
graph tw (scatter t prec, msymbol(O)) (function y= _b[_cons] + _b[prec]*x, range(0 6) lwidth(medthick) lcolor(black)), yline(0, lwidth(medthick) ) title("`pbias'% Publication Bias") name("fat_`trimming'", replace) xtitle("Precision") ytitle("t-value") legend(off)

local pet_t = _b[prec]/_se[prec]
local fat_t = _b[_cons]/_se[_cons]

*************************
*Caliper-Test
gen z = abs(invnormal(p_os))
local th = invnormal(0.975)

foreach num of numlist 3 5 10 15 {
local uc = `th' - ((`num'*0.01)*`th')
local oc = `th' + ((`num'*0.01)*`th')
gen cal`num' = 0 if z<=`th' & z > `uc'
replace cal`num' = 1 if z>`th' & z < `oc'

sum cal`num'
if r(N)==0 {
local ct`num'_p = -999
local ct`num'_N = -999
local ct`num'_P = -999
}

else {
bitest cal`num' = 0.5
local ct`num'_p = r(p_u)
local ct`num'_N = r(N)
local ct`num'_P = r(k)/r(N)
}

local drop uc oc th 
}

bitest cal10 = 0.5
hist z if z<4, width(0.098) start(0) xline(1.96, lwidth(medthick) ) freq xtitle("z-value") title("`pbias'% Publication Bias") name("ct5_`trimming'", replace) ylabel(0(20)120)
}

*************************
*Export Graphs
graph combine fat_0 fat_1, note("es=0.5, k=1000, n=100, file-drawer") xcommon ycommon
graph export example_fat.wmf, replace
graph combine pu_0 pu_1, note("es=0.5, k=1000, n=100, file-drawer") xcommon ycommon
graph export example_pu.wmf, replace
graph combine ct5_0 ct5_1, note("es=0.5, k=1000, n=100, file-drawer") xcommon ycommon
graph export example_ct5.wmf, replace
graph combine funnel_0 funnel_1, note("es=0.5, k=1000, n=100, file-drawer") xcommon ycommon
graph export example_funnel.wmf, replace
