
capture program drop dset
program define dset, rclass

syntax , [es(string)] [pb(string)] [trim(string)] [k(string)] [mu_n(string)] [het(string)]
matrix regs = J(`k',7,.)
matrix colnames regs = koef se N df pb_a pb_c pb

*********************
* no effect heterogeneity
if `het' == 0 {

forvalues run = 1(1)`k' {
global run = `run'

clear
local n = abs(round(rnormal(`mu_n',$sigma_n),1))
while `n' < 30 {							//trunctated normal at 100 700
local n = abs(round(rnormal(`mu_n',$sigma_n),1))
}

set obs `n'
if `es' == 0 {
gen x = rnormal(0, $sigma_x)
gen y = rnormal(0, $sigma_e)
}
else {
gen x = rnormal(0, $sigma_x)
gen y = `es'*x-rnormal(0, $sigma_e)
}

if `trim'==1{
trim_sim, pb(`pb')
}
if `trim'==0{
pb_sim, es(`es') pb(`pb') n(`n')
}
}
}

*********************
*effect heterogeneity
if `het' != 0 {
global run = 1		//def erste Zeile
local nk = (1-(`het'/100))*`k'

if `nk' > 0 {
forvalues run = 1(1)`nk' {
global run = `run'

clear
local n = abs(round(rnormal(`mu_n',$sigma_n),1)) 	//absolute values in order to avoid negative casenumber
while `n' <  30 {									//trunctated normal if smaller than 30
local n = abs(round(rnormal(`mu_n',$sigma_n),1))
}

set obs `n'											

if `es' == 0 {										//if zero effect
gen x = rnormal(0, $sigma_x)
gen y = rnormal(0, $sigma_e)
}

else {
gen x = rnormal(0, $sigma_x)						//set x-variable IDV
gen y = `es'*x+rnormal(0, $sigma_e)					//set y-variable DV
}

if `trim'==1{										//p-hacking simmulation process
trim_sim, pb(`pb') n(`n')
}
if `trim'==0{
pb_sim, es(`es') pb(`pb') n(`n')					//file-drawer simmulation process
}
}
}

scalar hk = `k'-`nk'

local hetrun = `nk'+1
forvalues run =  `hetrun'(1)`k' {					//heterogenous runs if het!=0|100
global run = `run'
local hds = runiform(0,$hetrange)

clear
local n = abs(round(rnormal(`mu_n',$sigma_n),1))	//absolute values in order to avoid negative casenumber
while `n' <  30 {									//trunctated normal if smaller than 30
local n = abs(round(rnormal(`mu_n',$sigma_n),1))
}

set obs `n'

gen x = rnormal(0, $sigma_x)						//set x-variable IDV
gen y = `hds'*x-rnormal(0, $sigma_e)				//set y-variable DV

if `trim'==1{
trim_sim, pb(`pb') n(`n')							//p-hacking simmulation process
}

if `trim'==0{										//file-drawer simmulation process
pb_sim, es(`hds') pb(`pb') n(`n')
}
}
}

local drop nk
end




*************************************
*PB-Simmulation (satisficing if p<0.05)
*************************************
*File-drawer
*************************************
capture program drop pb_sim
program define pb_sim
syntax [varlist], [es(string)] [pb(string)] [n(string)] 

fastreg y x, calcvcv
scalar reg_count = reg_count+1
matrix cvm = e(V)
matrix est = e(b)
scalar th = invttail(e(df_r),0.025)
scalar t = est[1,1]/(cvm[1,1]^0.5)

scalar random = runiform()

scalar pb_a =0
scalar pb_c =0

if `pb' > 0 {
scalar pb_th = `pb'/100


*Publication Bias
if random > pb_th {			//qualify for publication bias
}

else {
scalar pb_a = 1 // PB_Neigung
scalar pb_c = 0 // PB begangen


if t<=th {					// PB only if t<=th
					
forvalues sreg = 1(1)$ds {	// Loop over datasets
if t<=th {					// if t smaller threshold (e.g. 1.96)

clear
set obs `n'

if `es' == 0 {				//if effect size zero
gen x = rnormal(0, $sigma_x)
gen y = rnormal(0, $sigma_e)
}

else {
gen x = rnormal(0, $sigma_x)
gen y = `es'*x-rnormal(0, $sigma_e)
}

fastreg y x , calcvcv
scalar reg_count = reg_count+1
scalar pb_c = 1 			//PB committed

matrix t_cvm = e(V)
matrix t_est = e(b)
scalar ttemp = t_est[1,1]/(t_cvm[1,1]^0.5)
if ttemp>t {				// replace old result with new result if new result is better (in respect to t)
scalar t = ttemp
matrix cvm = t_cvm
matrix est = t_est
}
}
}
}
}
}

*Save study results
matrix regs[$run ,1] = est[1,1]
matrix regs[$run ,2]= cvm[1,1]^0.5
matrix regs[$run ,3] = e(N)
matrix regs[$run ,4] = e(df_r)
matrix regs[$run ,5] = pb_a
matrix regs[$run ,6] = pb_c
matrix regs[$run ,7] = `pb'

end


*************************************
*p-Hacking/trimming
capture program drop trim_sim
program define trim_sim
syntax [varlist], [es(string)] [pb(string)] [n(string)] 
*local pb = 50

fastreg y x, calcvcv
scalar reg_count = reg_count+1
matrix cvm = e(V)
matrix est = e(b)
scalar th = invttail(e(df_r),0.025)
scalar t = est[1,1]/(cvm[1,1]^0.5)
local df = e(df_r)

scalar random = runiform()
scalar pb_a =0
scalar pb_c =0

if `pb' > 0 {
scalar pb_th = `pb'/100


*Publication Bias
if random > pb_th {
}

else {
scalar pb_a = 1 
scalar pb_c = 0 



if t<=th {
forvalues coef = 1(1)$coefs { 					// generate covariate set
gen z`coef' = -0.5*y+0.5*x-rnormal(0, $sigma_z)	// collider setting because y and x causes z's
}
}


forvalues sreg = 1(1)$ntuples {					// all possible combinations of colliders
if t<=th {
fastreg y x ${tuple`sreg'}, calcvcv
scalar reg_count = reg_count+1
scalar pb_c = 1 // PB begangen
matrix t_cvm = e(V)
matrix t_est = e(b)
scalar ttemp = t_est[1,1]/(t_cvm[1,1]^0.5)
if ttemp>t {
scalar t = ttemp
matrix cvm = t_cvm
matrix est = t_est
local df = e(df_r)
}
}
}
}
}


*Matrix for saving study results
matrix regs[$run ,1] = est[1,1]
matrix regs[$run ,2]= cvm[1,1]^0.5
matrix regs[$run ,3] = e(N)
matrix regs[$run ,4] = e(df_r)
matrix regs[$run ,5] = pb_a
matrix regs[$run ,6] = pb_c
matrix regs[$run ,7] = `pb'


end
